# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import json
import requests

from typing import Dict, Any, Sequence

class Tadau():
  def __init__(self, params: Dict[str, Any] = {}):
    self.__api_url = 'https://www.google-analytics.com/mp/collect'
    self.__reserved_keys = ['app_instance_id',
                          'client_id', 'uuid', 'user_id', 'timestamp_micros']

    # Uses custom values, or defaults to standard
    self.api_secret = params.get('api_secret') or 'shyN6ac9RuyxdgLQZRIvyg'
    self.measurement_id = params.get('measurement_id') or 'G-SWYK2BS415'

  @staticmethod
  def _validate_param(key: str, value: Any, reserved_keys: Sequence[str]) -> bool:
    return key not in reserved_keys and value is not None and value != ''

  @staticmethod
  def _filter_alphanumeric_only(text: str) -> str:
      return re.sub(r'[^a-zA-Z0-9-]', '', text)

  def process(self, events: Any):
    try:
      for row in events:
        client_id = row.get('client_id')

        # client_id is a required attribute. Therefore, skips if not set
        if not client_id:
          continue

        user_id = row.get('user_id')

        # Sets client_id with an empty payload
        payload: Dict[str, Any] = {
            'non_personalized_ads': True,
            'client_id': f"{client_id}",
        }

        # Only adds params that aren't reserved keywords
        event_reserved_keys = self.__reserved_keys + ['name']
        params = {k: v for k, v in row.items() if self._validate_param(k,
                                                                      v, event_reserved_keys)}
        payload['events'] = [
            {'name': self._filter_alphanumeric_only(row['name']), 'params': params}]

        # Sets user_id if any
        if user_id:
          payload['user_id'] = f"{user_id}"

        # Sends to GA4
        requests.post(
            f'{self.__api_url}?api_secret={self.api_secret}&measurement_id={self.measurement_id}', data=json.dumps(payload))
    except:
      pass



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import requests_mock

from tadau.measurement_protocol import Tadau

@pytest.fixture
def uploader():
    return Tadau()

def test_event_name_validation(uploader):
    with requests_mock.Mocker() as m:
        m.post(requests_mock.ANY, status_code=204)

        uploader.process([{
           'client_id': '123',
            'name': 'event name 1',
            'value': '42',
            'important_event': 'False',
            'user_id': '11'
        }])

        assert m.last_request.json()['events'][0]['name'] == 'eventname1'

def test_fail_safe_with_required_params(uploader):
    with requests_mock.Mocker() as m:
        m.post(requests_mock.ANY, status_code=204)

        uploader.process([{
            'client_id': '123',
            'name': 'event_name',
            'value': '42',
            'important_event': 'False',
            'user_id': '11'
        }, {
            'name': 'event_name',
            'value': '42',
            'important_event': 'False',
        }])

        assert m.call_count == 1

def test_succesful_triggered_events(uploader):
    with requests_mock.Mocker() as m:
        m.post(requests_mock.ANY, status_code=204)

        uploader.process([{
           'client_id': '123',
            'name': 'event_name',
            'value': '42',
            'important_event': 'False',
            'user_id': '11'
        }, {
            'client_id': '4321',
            'name': 'event_name',
            'value': '42',
            'important_event': 'False',
        }])

        assert m.call_count == 2
        assert 'user_id' not in m.last_request.json().keys()



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import io

install_requires = []

with io.open("README.rst", "r", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

setup(
    name="tadau",
    version="1.0.1",
    author="Google LLC",
    author_email="gps-latam-solutions@googlegroups.com",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    description="Tadau offers a sample on how to TrAck Downloads, Adaption and Usage of Solutions",
    include_package_data=True,
    python_requires=">=3.7",
    long_description=long_description,
    install_requires=install_requires,
    extras_require={
    },
    license="Apache 2.0",
    packages=find_packages(
        exclude=["examples", "examples.*", "tests", "tests.*"]
    ),
    namespace_packages=["tadau"],
    url="https://github.com/google/tadau",
)
